#pragma once

#include <ChamberComponent.h>

#include <string>
#include <iostream>

class BackgroundVisitor;

/** Base class for the Leafs of the Composite.
 */
class Cell : public ChamberComponent {
public:
  /** Constructs an empty drift cell with given position as lower left corner.
   */
  Cell(unsigned xPosition, unsigned yPosition, ChamberComponent* parent) :
    ChamberComponent(parent),
    m_xPosition(xPosition), m_yPosition(yPosition),
    m_eDepositionCount(0)
  {}

  /** Virtual destructor, as this is intended as a base class.
  */
  virtual ~Cell() {};

  /** SpecialCells overwrite this to know, if the given x is within their boundary.
   */
  virtual bool xIsInside(unsigned x) = 0;

  /** Visitor Design Pattern function allowing Visitors to add background.
   */
  virtual void accept(BackgroundVisitor& backgroundVisitor);

  /** Adds a given amount of energy to the cell.
   */
  void addEDeposition(unsigned eDepositionCount)
  {
    m_eDepositionCount += eDepositionCount;
  }

  /** Returns a counter of the energy deposition.
   */
  unsigned getEDeposition()
  {
    return m_eDepositionCount;
  }

  /** A string to be printed in context.
   *
   *  In this case a black background with an O for an empty cell
   *  and an X for a hit cell is printed.
   */
  virtual std::string visualize() = 0;

  /** Function to return the next Cell; designed for Iterators over all Cells.
   */
  Cell* getNextCell()
  {
    return dynamic_cast<Cell*>(m_parent->getNextChild(this));
  }

protected:
  unsigned m_xPosition;
  unsigned m_yPosition;

  unsigned m_eDepositionCount;
};
