#pragma once

#include <memory>

#include <SpecialCell.h>
#include <ChamberIterator.h>
#include <ChamberComponent.h>
#include <ColourStrategy.h>
#include <SuperLayer.h>

/** Top Level class of the Composite.
 */
class CompositeChamber : public ChamberComponent {
public:
  /** Constructor determining the width of each layer.
   *
   *  As this is the top level of the Composite, there is no parent to point to.
   */
  CompositeChamber(unsigned xSizeCounter = 0) :
    ChamberComponent(nullptr),
    m_xSizeCounter(xSizeCounter)
  {}

  /** Getter for a ChamberIterator to the first Cell.
   */
  ChamberIterator first()
  {
    return ChamberIterator(dynamic_cast<Cell*>(deepFirst()));
  }

  /** Overriding the visualize function to add some frame.
   */
  std::string visualize() override
  {
    std::cout << std::endl;
    for (unsigned ii = 0; ii < m_xSizeCounter; ii++) {
      std::cout << "-";
    }
    std::cout << std::endl;
    ChamberComponent::visualize();
    std::cout << std::endl;
    for (unsigned ii = 0; ii < m_xSizeCounter; ii++) {
      std::cout << "-";
    }
    std::cout << std::endl;
    return "";
  }

  /** Adding a SuperLayer, which is necessarily the next Component in the tree of the Composite.
   */
  template <typename ColourType>
  SuperLayer<ColourType>* addSuperLayer()
  {
    m_children.emplace_back(new SuperLayer<ColourType>(this));
    return (static_cast<SuperLayer<ColourType>*>(m_children.back()));
  }

  /** Getter for width of the Chamber.
   */
  unsigned getMaxX() const override
  {
    return m_xSizeCounter;
  }

private:
  unsigned m_xSizeCounter;
};
