#pragma once

#include <Cell.h>
#include <ColourStrategy.h>
#include <WidthStrategy.h>

# include <memory>

template <typename WidthType, typename ColourType>
class SpecialCell : public Cell, private WidthType, ColourType {
  using WidthType::getWidth;
  using WidthType::getEmptyDisplayElement;
  using WidthType::getHitDisplayElement;

  using ColourType::getColourString;
public:
  /** Constructor with position and strategies.
   *
   *  Note that in the current version
   */
  SpecialCell(unsigned xPosition, unsigned yPosition,
              ChamberComponent* parent) :
    Cell(xPosition, yPosition, parent)
  {}

  bool xIsInside(unsigned x) override
  {
    if (x >=  m_xPosition && x <= m_xPosition + getWidth()) {
      return true;
    }
    return false;
  }

  unsigned getMaxY() const override
  {
    return 0;
  }

  std::string visualize() override
  {
    std::string returnString = getColourString();
    if (m_eDepositionCount == 0) {
      returnString += getEmptyDisplayElement();
    } else {
      returnString += getHitDisplayElement();
    }
    return (returnString += "\x1B[0m");
  }
};
