#pragma once

#include <ChamberComponent.h>
#include <Layer.h>
#include <ColourStrategy.h>
#include <WidthStrategy.h>

#include <memory>

/** SuperLayers are the Components below the complete CompositeChamber and above the single Layers.
 *
 *  The ColourStrategy is defined by the SuperLayer, e.g. all Layers in a SuperLayer have the same
 *  colour.
 */
template <typename ColourType>
class SuperLayer : public ChamberComponent {
public:
  /** Constructor assigning a CompositeChamber as parent and a ColourStrategy.
   */
  SuperLayer(ChamberComponent* parent) :
    ChamberComponent(parent)
  {}

  /** Adding a Layer as daughter to the SuperLayer and defining the width of each Cell in the Layer.
   *
   *  All Cells in a Layer have the same width.
   */
  template <typename WidthType>
  Layer<WidthType, ColourType>*  addLayer()
  {
    m_children.emplace_back(new Layer<WidthType, ColourType>(this));
    return dynamic_cast<Layer<WidthType, ColourType>*>(m_children.back());
  }
};
